@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXCursor.c,v 1.2 2002/08/31 06:12:29 das Exp $
+ * RCS: @(#) $Id: tkMacOSXCursor.c,v 1.4 2003/02/11 07:26:18 wolfsuit Exp $
  */
 
 #include "tkPort.h"
@@ -20,10 +20,14 @@
 
 /*
  * There are three different ways to set the cursor on the Mac.
+ * The default theme cursors (listed in cursorNames below),
+ * color resource cursors, & normal cursors.
  */
-#define ARROW        0        /* The arrow cursor. */
-#define COLOR        1        /* Cursors of type crsr. */
-#define NORMAL        2        /* Cursors of type CURS. */
+
+#define THEME        0        /* Theme cursors */
+#define ANIMATED     1        /* Animated theme cursors */
+#define COLOR        2        /* Cursors of type crsr. */
+#define NORMAL       3        /* Cursors of type CURS. */
 
 /*
  * The following data structure contains the system specific data
@@ -32,8 +36,13 @@
 
 typedef struct {
     TkCursor info;                /* Generic cursor info used by tkCursor.c */
-    Handle macCursor;                /* Resource containing Macintosh cursor. */
-    int type;                        /* Type of Mac cursor: arrow, crsr, CURS */
+    Handle macCursor;                /* Resource containing Macintosh cursor. 
+                                      * For theme cursors, this is -1. */
+    int type;                        /* Type of Mac cursor: for theme cursors
+                                      * this is the theme cursor constant,
+                                      * otherwise one of crsr or CURS */
+    unsigned int count;              /* For animating cursors, the count for the
+                                        cursor. */
 } TkMacOSXCursor;
 
 /*
@@ -41,19 +50,32 @@ typedef struct {
  * to its resource identifier.
  */
 
-static struct CursorName {
+struct CursorName {
     char *name;
     int id;
-} cursorNames[] = {
-    {"ibeam",                1},
-    {"text",                1},
-    {"xterm",                1},
-    {"cross",                2},
-    {"crosshair",        2},
-    {"cross-hair",        2},
-    {"plus",                3},
-    {"watch",                4},
-    {"arrow",                5},
+};
+
+static struct CursorName themeCursorNames[] = {
+    {"ibeam",                kThemeIBeamCursor},
+    {"text",                kThemeIBeamCursor},
+    {"xterm",                kThemeIBeamCursor},
+    {"cross",                kThemeCrossCursor},
+    {"crosshair",        kThemeCrossCursor},
+    {"cross-hair",        kThemeCrossCursor},
+    {"plus",                kThemePlusCursor},
+    {"arrow",                kThemeArrowCursor},
+    {"closedhand",	kThemeClosedHandCursor},
+    {"openhand",	kThemeOpenHandCursor},
+    {"pointinghand",	kThemePointingHandCursor},
+    {NULL,                0}
+};
+
+static struct CursorName animatedThemeCursorNames[] = {
+    {"watch",                kThemeWatchCursor},
+    {"countinguphand",	kThemeCountingUpHandCursor},
+    {"countingdownhand",	kThemeCountingDownHandCursor},
+    {"countingupanddownhand",	kThemeCountingUpAndDownHandCursor},
+    {"spinning",	kThemeSpinningCursor},
     {NULL,                0}
 };
 
@@ -163,14 +185,15 @@ FindCursorByName(
 
 TkCursor *
 TkGetCursorByName(
-    Tcl_Interp *interp,                /* Interpreter to use for error reporting. */
-    Tk_Window tkwin,                /* Window in which cursor will be used. */
-    Tk_Uid string)                /* Description of cursor.  See manual entry
-                                 * for details on legal syntax. */
+    Tcl_Interp *interp,      /* Interpreter to use for error reporting. */
+    Tk_Window tkwin,         /* Window in which cursor will be used. */
+    Tk_Uid string)           /* Description of cursor.  See manual entry
+                              * for details on legal syntax. */
 {
     struct CursorName *namePtr;
     TkMacOSXCursor *macCursorPtr;
-
+    int count = -1;
+    
     macCursorPtr = (TkMacOSXCursor *) ckalloc(sizeof(TkMacOSXCursor));
     macCursorPtr->info.cursor = (Tk_Cursor) macCursorPtr;
 
@@ -180,22 +203,41 @@ TkGetCursorByName(
      * attempt to load the cursor as a named Mac resource.
      */
 
-    for (namePtr = cursorNames; namePtr->name != NULL; namePtr++) {
+    for (namePtr = themeCursorNames; namePtr->name != NULL; namePtr++) {
         if (strcmp(namePtr->name, string) == 0) {
+            macCursorPtr->count = -1;
+            macCursorPtr->macCursor = (Handle) namePtr;
+            macCursorPtr->type = THEME;
             break;
         }
     }
 
-
-    if (namePtr->name != NULL) {
-            if (namePtr->id == 5) {
-            macCursorPtr->macCursor = (Handle) -1;
-            macCursorPtr->type = ARROW;
-            } else {
-            macCursorPtr->macCursor = (Handle) GetCursor(namePtr->id);
-            macCursorPtr->type = NORMAL;
+    if (namePtr->name == NULL) {
+        for (namePtr = animatedThemeCursorNames;
+                namePtr->name != NULL; namePtr++) {
+            int namelen = strlen (namePtr->name);
+            if (strncmp(namePtr->name, string, namelen) == 0) {
+                const char *numPtr = string + namelen;
+                if (*numPtr == '\0') {
+                    count = -1;
+                } else {
+                    int result;
+                    result = Tcl_GetInt(NULL, numPtr, &count);
+                    if (result != TCL_OK) {
+                        continue;
+                    }
+                }
+                macCursorPtr->macCursor = (Handle) namePtr;
+                macCursorPtr->type = ANIMATED;
+                macCursorPtr->count = count;
+                break;
+            }
         }
-    } else {
+    }
+        
+
+
+    if (namePtr->name == NULL) {
         FindCursorByName(macCursorPtr, string);
 
         if (macCursorPtr->macCursor == NULL) {
@@ -294,7 +336,6 @@ TkpFreeCursor(
         gCurrentCursor = NULL;
     }
 }
-
 /*
  *----------------------------------------------------------------------
  *
@@ -320,6 +361,7 @@ TkMacOSXInstallCursor(
     TkMacOSXCursor *macCursorPtr = gCurrentCursor;
     CCrsrHandle ccursor;
     CursHandle cursor;
+    static unsigned int cursorStep = 0;
     
     gResizeOverride = resizeOverride;
 
@@ -332,10 +374,24 @@ TkMacOSXInstallCursor(
             fprintf(stderr,"Resize cursor failed, %d\n", ResError());
              */
         }
-    } else if (macCursorPtr == NULL || macCursorPtr->type == ARROW) {
+    } else if (macCursorPtr == NULL) {
         SetThemeCursor(kThemeArrowCursor);
     } else {
+        struct CursorName *namePtr;
         switch (macCursorPtr->type) {
+            case THEME:
+                namePtr = (struct CursorName *) macCursorPtr->macCursor;
+                SetThemeCursor(
+                        namePtr->id);
+                break;
+            case ANIMATED:
+                namePtr = (struct CursorName *) macCursorPtr->macCursor;
+                if (macCursorPtr->count == -1) {
+                    SetAnimatedThemeCursor(namePtr->id, cursorStep++);
+                } else {
+                    SetAnimatedThemeCursor(namePtr->id, macCursorPtr->count);
+                }
+                break;
             case COLOR:
                 ccursor = (CCrsrHandle) macCursorPtr->macCursor;
                 SetCCursor(ccursor);
@@ -368,16 +424,25 @@ void
 TkpSetCursor(
     TkpCursor cursor)
 {
+    int cursorChanged = 1;
+    
     if (!gTkOwnsCursor) {
         return;
     }
+    
     if (cursor == None) {
+        if (gCurrentCursor == NULL) {
+            cursorChanged = 0;
+        }
         gCurrentCursor = NULL;
     } else {
+        if (gCurrentCursor == (TkMacOSXCursor *) cursor) {
+            cursorChanged = 0;
+        }
         gCurrentCursor = (TkMacOSXCursor *) cursor;
     }
 
-    if (Tk_MacOSXIsAppInFront()) {
+    if (Tk_MacOSXIsAppInFront() && cursorChanged) {
         TkMacOSXInstallCursor(gResizeOverride);
     }
 }
